var Matrix;
var __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
  for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
  function ctor() { this.constructor = child; }
  ctor.prototype = parent.prototype;
  child.prototype = new ctor;
  child.__super__ = parent.prototype;
  return child;
};
Matrix = require('./matrix');
module.exports = function(numeric) {
  var GenericMatrix;
  return GenericMatrix = (function() {
    var decompose;
    __extends(GenericMatrix, Matrix);
    function GenericMatrix() {
      GenericMatrix.__super__.constructor.apply(this, arguments);
    }
    GenericMatrix.identity = function(n) {
      var i, items, j;
      items = [];
      for (i = 0; i < n; i += 1) {
        for (j = 0; j < n; j += 1) {
          items.push(i === j ? numeric.unit : numeric.zero);
        }
      }
      return new this(n, n, items);
    };
    GenericMatrix.permutation = function(perms, inverted) {
      var i, items, j, n;
      if (inverted == null) {
        inverted = false;
      }
      items = [];
      n = perms.length;
      for (i = 0; i < n; i += 1) {
        for (j = 0; j < n; j += 1) {
          if (inverted) {
            items.push(perms[i] === j ? numeric.unit : numeric.zero);
          } else {
            items.push(perms[j] === i ? numeric.unit : numeric.zero);
          }
        }
      }
      return new this(n, n, items);
    };
    decompose = function() {
      var det, i, ii, ind, j, k, lower, pivot, pivotVal, prod, row, size, temp, test, testVal, upper, _ref, _ref2;
      if (this.rows !== this.cols) {
        throw new Error('Dimensionality mismatch');
      }
      size = this.rows;
      upper = this.clone();
      lower = this.constructor.identity(size);
      row = (function() {
        var _results;
        _results = [];
        for (i = 0; i < size; i += 1) {
          _results.push(i);
        }
        return _results;
      })();
      det = numeric.unit;
      for (i = 0; i < size; i += 1) {
        pivot = i;
        pivotVal = upper.get(row[pivot], i);
        for (j = _ref = i + 1; j < size; j += 1) {
          test = j;
          testVal = upper.get(row[test], i);
          if (numeric.abs(testVal) > numeric.abs(pivotVal)) {
            pivotVal = testVal;
            pivot = test;
          }
        }
        det = numeric.mul(det, pivotVal);
        if (numeric.eq(pivotVal, numeric.zero)) {
          return {
            det: numeric.zero
          };
        }
        if (i !== pivot) {
          temp = row[i];
          row[i] = row[pivot];
          row[pivot] = temp;
          det = numeric.neg(det);
        }
        for (j = 0; j < size; j += 1) {
          ind = upper.index(row[i], j);
          upper.items[ind] = numeric.div(upper.items[ind], pivotVal);
          lower.items[ind] = numeric.div(lower.items[ind], pivotVal);
        }
        for (ii = _ref2 = i + 1; ii < size; ii += 1) {
          k = upper.get(row[ii], i);
          for (j = 0; j < size; j += 1) {
            ind = upper.index(row[ii], j);
            prod = numeric.mul(k, upper.get(row[i], j));
            upper.items[ind] = numeric.sub(upper.items[ind], prod);
            prod = numeric.mul(k, lower.get(row[i], j));
            lower.items[ind] = numeric.sub(lower.items[ind], prod);
          }
        }
      }
      return {
        upper: upper,
        lower: lower,
        perm: row,
        det: det
      };
    };
    GenericMatrix.prototype.det = function() {
      return decompose.call(this).det;
    };
    GenericMatrix.prototype.inv = function() {
      var det, i, ii, ind, j, k, lower, perm, prod, res, size, upper, _ref, _ref2, _ref3;
      _ref = decompose.call(this), upper = _ref.upper, lower = _ref.lower, perm = _ref.perm, det = _ref.det;
      if (numeric.abs(det) < 1e-8) {
        throw new Error('@constructor is not invertible or is ill-conditioned');
      }
      size = this.rows;
      for (i = _ref2 = size - 1; i >= 0; i += -1) {
        for (ii = _ref3 = i - 1; ii >= 0; ii += -1) {
          k = upper.get(perm[ii], i);
          for (j = 0; j < size; j += 1) {
            ind = upper.index(perm[ii], j);
            prod = numeric.mul(k, upper.get(perm[i], j));
            upper.items[ind] = numeric.sub(upper.items[ind], prod);
            prod = numeric.mul(k, lower.get(perm[i], j));
            lower.items[ind] = numeric.sub(lower.items[ind], prod);
          }
        }
      }
      res = new this.constructor(size, size);
      for (i = 0; i < size; i += 1) {
        for (j = 0; j < size; j += 1) {
          res.items[res.index(i, j)] = lower.items[lower.index(perm[i], j)];
        }
      }
      return res;
    };
    GenericMatrix.prototype.neg = function() {
      var item;
      return new this.constructor(this.rows, this.cols, (function() {
        var _i, _len, _ref, _results;
        _ref = this.items;
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          item = _ref[_i];
          _results.push(numeric.neg(item));
        }
        return _results;
      }).call(this));
    };
    GenericMatrix.prototype.add = function(other) {
      var i, items, _ref;
      if (this.rows !== other.rows || this.cols !== other.cols) {
        throw new Error('Dimensionality mismatch');
      }
      items = [];
      for (i = 0, _ref = this.items.length; i < _ref; i += 1) {
        items.push(numeric.add(this.items[i], other.items[i]));
      }
      return new this.constructor(this.rows, this.cols, items);
    };
    GenericMatrix.prototype.mul = function(other) {
      var i, items, j, k, sum, _ref, _ref2, _ref3;
      if (this.cols !== other.rows) {
        throw new Error('Dimensionality mismatch');
      }
      items = [];
      for (i = 0, _ref = this.rows; i < _ref; i += 1) {
        for (j = 0, _ref2 = other.cols; j < _ref2; j += 1) {
          sum = numeric.zero;
          for (k = 0, _ref3 = this.cols; k < _ref3; k += 1) {
            sum = numeric.add(sum, numeric.mul(this.get(i, k), other.get(k, j)));
          }
          items[i * other.cols + j] = sum;
        }
      }
      return new this.constructor(this.rows, other.cols, items);
    };
    GenericMatrix.prototype.norm = function() {
      var item, norm, _i, _len, _ref;
      norm = 0;
      _ref = this.items;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        item = _ref[_i];
        norm += numeric.mul(item, item);
      }
      return numeric.abs(norm) / this.items.length;
    };
    GenericMatrix.prototype.toString = function() {
      var ind, item, items, row, s, slice, _ref;
      s = '';
      for (row = 0, _ref = this.rows; row < _ref; row += 1) {
        ind = row * this.cols;
        slice = this.items.slice(ind, ind + this.cols);
        items = (function() {
          var _i, _len, _results;
          _results = [];
          for (_i = 0, _len = slice.length; _i < _len; _i++) {
            item = slice[_i];
            _results.push(numeric.abs(item).toFixed(2));
          }
          return _results;
        })();
        s += items.join(', ') + "\n";
      }
      return s;
    };
    return GenericMatrix;
  })();
};