var Matrix;
var __slice = Array.prototype.slice;
module.exports = Matrix = (function() {
  var decompose;
  function Matrix(rows, cols, items) {
    if (items == null) {
      items = [];
    }
    this.rows = rows;
    this.cols = cols;
    this.items = items;
  }
  Matrix.identity = function(n) {
    var i, items, j;
    items = [];
    for (i = 0; i < n; i += 1) {
      for (j = 0; j < n; j += 1) {
        items.push(i === j ? 1 : 0);
      }
    }
    return new this(n, n, items);
  };
  Matrix.permutation = function(perms, inverted) {
    var i, items, j, n;
    if (inverted == null) {
      inverted = false;
    }
    items = [];
    n = perms.length;
    for (i = 0; i < n; i += 1) {
      for (j = 0; j < n; j += 1) {
        if (inverted) {
          items.push(perms[i] === j ? 1 : 0);
        } else {
          items.push(perms[j] === i ? 1 : 0);
        }
      }
    }
    return new this(n, n, items);
  };
  Matrix.prototype.index = function(i, j) {
    return i * this.cols + j;
  };
  Matrix.prototype.get = function(i, j) {
    return this.items[this.index(i, j)];
  };
  Matrix.prototype.proj = function() {
    var fn, fns, ind, items, row, _i, _len, _ref;
    fns = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
    items = [];
    for (row = 0, _ref = this.rows; row < _ref; row += 1) {
      for (_i = 0, _len = fns.length; _i < _len; _i++) {
        fn = fns[_i];
        ind = row * this.cols;
        items.push(fn(this.items.slice(ind, ind + this.cols)));
      }
    }
    return new this.constructor(this.rows, fns.length, items);
  };
  Matrix.prototype.t = function() {
    var i, items, j, _ref, _ref2;
    items = [];
    for (i = 0, _ref = this.rows; i < _ref; i += 1) {
      for (j = 0, _ref2 = this.cols; j < _ref2; j += 1) {
        items[j * this.rows + i] = this.get(i, j);
      }
    }
    return new this.constructor(this.cols, this.rows, items);
  };
  Matrix.prototype.clone = function() {
    return new this.constructor(this.rows, this.cols, this.items.slice(0));
  };
  Matrix.prototype.augment = function(other) {
    var otherT, selfT, _ref;
    if (this.rows !== other.rows) {
      throw new Error('Dimensionality mismatch');
    }
    selfT = this.t();
    otherT = other.t();
    selfT.rows += otherT.rows;
    (_ref = selfT.items).push.apply(_ref, otherT.items);
    return selfT.t();
  };
  Matrix.prototype.sliceRows = function(begin, end) {
    if (begin < 0) {
      begin += this.rows;
    }
    if (begin < 0) {
      begin = 0;
    }
    if (end == null) {
      end = this.rows;
    }
    if (end > this.rows) {
      end = this.rows;
    } else if (end < 0) {
      end += this.rows;
    }
    if (end < begin) {
      end = begin;
    }
    return new this.constructor(end - begin, this.cols, this.items.slice(begin * this.cols, end * this.cols));
  };
  Matrix.prototype.sliceCols = function(begin, end) {
    if (end == null) {
      end = this.cols;
    }
    return this.t().sliceRows(begin, end).t();
  };
  Matrix.prototype.slice = function(beginCols, beginRows, endCols, endRows) {
    if (beginRows == null) {
      beginRows = 0;
    }
    if (endCols == null) {
      endCols = this.cols;
    }
    if (endRows == null) {
      endRows = this.rows;
    }
    return this.sliceRows(beginRows, endRows).sliceCols(beginCols, endCols);
  };
  decompose = function() {
    var det, i, ii, ind, j, k, lower, pivot, pivotVal, row, size, temp, test, testVal, upper, _ref, _ref2;
    if (this.rows !== this.cols) {
      throw new Error('Dimensionality mismatch');
    }
    size = this.rows;
    upper = this.clone();
    lower = this.constructor.identity(size);
    row = (function() {
      var _results;
      _results = [];
      for (i = 0; i < size; i += 1) {
        _results.push(i);
      }
      return _results;
    })();
    det = 1;
    for (i = 0; i < size; i += 1) {
      pivot = i;
      pivotVal = upper.get(row[pivot], i);
      for (j = _ref = i + 1; j < size; j += 1) {
        test = j;
        testVal = upper.get(row[test], i);
        if (Math.abs(testVal) > Math.abs(pivotVal)) {
          pivotVal = testVal;
          pivot = test;
        }
      }
      det *= pivotVal;
      if (pivotVal === 0) {
        return {
          det: 0
        };
      }
      if (i !== pivot) {
        temp = row[i];
        row[i] = row[pivot];
        row[pivot] = temp;
        det = -det;
      }
      for (j = 0; j < size; j += 1) {
        ind = upper.index(row[i], j);
        upper.items[ind] /= pivotVal;
        lower.items[ind] /= pivotVal;
      }
      for (ii = _ref2 = i + 1; ii < size; ii += 1) {
        k = upper.get(row[ii], i);
        for (j = 0; j < size; j += 1) {
          ind = upper.index(row[ii], j);
          upper.items[ind] -= k * upper.get(row[i], j);
          lower.items[ind] -= k * lower.get(row[i], j);
        }
      }
    }
    return {
      upper: upper,
      lower: lower,
      perm: row,
      det: det
    };
  };
  Matrix.prototype.det = function() {
    return decompose.call(this).det;
  };
  Matrix.prototype.inv = function() {
    var det, i, ii, ind, j, k, lower, perm, res, size, upper, _ref, _ref2, _ref3;
    _ref = decompose.call(this), upper = _ref.upper, lower = _ref.lower, perm = _ref.perm, det = _ref.det;
    if ((-1e-8 < det && det < 1e-8)) {
      throw new Error('Matrix is not invertible or is ill-conditioned');
    }
    size = this.rows;
    for (i = _ref2 = size - 1; i >= 0; i += -1) {
      for (ii = _ref3 = i - 1; ii >= 0; ii += -1) {
        k = upper.get(perm[ii], i);
        for (j = 0; j < size; j += 1) {
          ind = upper.index(perm[ii], j);
          upper.items[ind] -= k * upper.get(perm[i], j);
          lower.items[ind] -= k * lower.get(perm[i], j);
        }
      }
    }
    res = new this.constructor(size, size);
    for (i = 0; i < size; i += 1) {
      for (j = 0; j < size; j += 1) {
        res.items[res.index(i, j)] = lower.items[lower.index(perm[i], j)];
      }
    }
    return res;
  };
  Matrix.prototype.neg = function() {
    var item;
    return new this.constructor(this.rows, this.cols, (function() {
      var _i, _len, _ref, _results;
      _ref = this.items;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        item = _ref[_i];
        _results.push(-item);
      }
      return _results;
    }).call(this));
  };
  Matrix.prototype.add = function(other) {
    var i, items, _ref;
    if (this.rows !== other.rows || this.cols !== other.cols) {
      throw new Error('Dimensionality mismatch');
    }
    items = [];
    for (i = 0, _ref = this.items.length; i < _ref; i += 1) {
      items.push(this.items[i] + other.items[i]);
    }
    return new this.constructor(this.rows, this.cols, items);
  };
  Matrix.prototype.sub = function(other) {
    return this.add(other.neg());
  };
  Matrix.prototype.mul = function(other) {
    var i, items, j, k, sum, _ref, _ref2, _ref3;
    if (this.cols !== other.rows) {
      throw new Error('Dimensionality mismatch');
    }
    items = [];
    for (i = 0, _ref = this.rows; i < _ref; i += 1) {
      for (j = 0, _ref2 = other.cols; j < _ref2; j += 1) {
        sum = 0;
        for (k = 0, _ref3 = this.cols; k < _ref3; k += 1) {
          sum += this.get(i, k) * other.get(k, j);
        }
        items[i * other.cols + j] = sum;
      }
    }
    return new this.constructor(this.rows, other.cols, items);
  };
  Matrix.prototype.div = function(other) {
    return this.mul(other.inv());
  };
  Matrix.prototype.norm = function() {
    var item, norm, _i, _len, _ref;
    norm = 0;
    _ref = this.items;
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      item = _ref[_i];
      norm += item * item;
    }
    return norm / this.items.length;
  };
  Matrix.prototype.eq = function(other, eps) {
    if (eps == null) {
      eps = 0;
    }
    return this.sub(other).norm() <= eps;
  };
  Matrix.prototype.toString = function() {
    var ind, item, items, row, s, _ref;
    s = '';
    for (row = 0, _ref = this.rows; row < _ref; row += 1) {
      ind = row * this.cols;
      items = (function() {
        var _i, _len, _ref2, _results;
        _ref2 = this.items.slice(ind, ind + this.cols);
        _results = [];
        for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
          item = _ref2[_i];
          _results.push(item.toFixed(2));
        }
        return _results;
      }).call(this);
      s += items.join(', ') + "\n";
    }
    return s;
  };
  return Matrix;
})();